'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _QueryBuilder = require('./QueryBuilder');

var _QueryBuilder2 = _interopRequireDefault(_QueryBuilder);

var _config = require('./config');

var _bunchOfExceptions = require('bunch-of-exceptions');

var _ResponseHandler = require('./ResponseHandler');

var _ResponseHandler2 = _interopRequireDefault(_ResponseHandler);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Model = function () {
    function Model() {
        (0, _classCallCheck3.default)(this, Model);

        this.queryBuilder = new _QueryBuilder2.default();
        this.selfValidate();
        this.type = this.getNamespace();

        this.parent = null;

        this.fields = this.getFields();
        this.relations = this.getRelations();
        this.dates = this.getDates();
        this.namespace = this.getNamespace();
    }

    // override

    (0, _createClass3.default)(Model, [{
        key: 'getFields',
        value: function getFields() {
            throw (0, _bunchOfExceptions.UnimplementedException)("Method getFields() must be implemented in every child class.");
        }
    }, {
        key: 'getDates',
        value: function getDates() {
            return {};
        }
    }, {
        key: 'getRelations',
        value: function getRelations() {
            return {};
        }
    }, {
        key: 'computed',
        value: function computed() {
            return {};
        }
    }, {
        key: 'getNamespace',
        value: function getNamespace() {
            throw (0, _bunchOfExceptions.UnimplementedException)("Method getNamespace() must be implemented in every child class.");
        }
    }, {
        key: 'getBaseUrl',
        value: function getBaseUrl() {
            return _config.REST_CONFIG.base_url;
        }

        /**
         * Make changes or use fetch config before the request
         * @param fetchConfig the config
         * @returns the changed fetch config
         */

    }, {
        key: 'beforeFetch',
        value: function beforeFetch(fetchConfig) {
            return fetchConfig;
        }

        /**
         * Make changes or use fetch response before the jsoning
         * @param fetchResponse the fetch response
         * @returns the changed response
         */

    }, {
        key: 'afterFetch',
        value: function afterFetch(fetchResponse) {
            return fetchResponse;
        }
    }, {
        key: 'toJson',
        value: function toJson() {
            var json = {};

            var fields = this.getFields();
            var relations = this.getRelations();
            var relationsName = Object.keys(relations);
            var dates = this.getDates();
            var datesName = Object.keys(dates);

            for (var prop in this) {
                if (fields.includes(prop)) {
                    json[prop] = this[prop];
                } else if (datesName.includes(prop)) {
                    json[prop] = this[prop].toString();
                    if (json[prop].includes('T')) {
                        json[prop] = json[prop].replace('T', ' ').replace('Z', '');
                    }
                } else if (relationsName.includes(prop)) {
                    if (relations[prop].list) {
                        json[prop] = this[prop].map(function (one) {
                            return one.toJson();
                        });
                    } else {
                        json[prop] = this[prop].toJson();
                    }
                }
            }

            return json;
        }

        /**
         * Lazy loads relations to the model.
         * Does not load already loaded relations.
         * To force the load, use Model#forceLazyLoad() method
         *
         * @param relations a list of relations to load. The function doesn't reload an already loaded relation.
         * @returns {Promise<*>} formatted loaded relation if single, otherwise returns this.
         */

    }, {
        key: 'lazyLoad',
        value: function () {
            var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
                for (var _len = arguments.length, relations = Array(_len), _key = 0; _key < _len; _key++) {
                    relations[_key] = arguments[_key];
                }

                return _regenerator2.default.wrap(function _callee$(_context) {
                    while (1) {
                        switch (_context.prev = _context.next) {
                            case 0:
                                relations = this.filterLoadedRelations.apply(this, (0, _toConsumableArray3.default)(relations));
                                return _context.abrupt('return', this.forceLazyLoad.apply(this, (0, _toConsumableArray3.default)(relations)));

                            case 2:
                            case 'end':
                                return _context.stop();
                        }
                    }
                }, _callee, this);
            }));

            function lazyLoad() {
                return _ref.apply(this, arguments);
            }

            return lazyLoad;
        }()

        /**
         * Lazy loads relations to the model.
         * This function FORCE all relations to be loaded, even the already loaded ones.
         * To avoid the force reload, use Model#lazyLoad() method
         *
         * @param relations a list of relations to load.
         * @returns {Promise<*>} this.
         */

    }, {
        key: 'forceLazyLoad',
        value: function () {
            var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2() {
                for (var _len2 = arguments.length, relations = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
                    relations[_key2] = arguments[_key2];
                }

                var _ref3, newThis, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, r, _iteratorNormalCompletion2, _didIteratorError2, _iteratorError2, _iterator2, _step2, _r;

                return _regenerator2.default.wrap(function _callee2$(_context2) {
                    while (1) {
                        switch (_context2.prev = _context2.next) {
                            case 0:
                                if (!(relations.length > 0)) {
                                    _context2.next = 49;
                                    break;
                                }

                                _context2.prev = 1;
                                _context2.next = 4;
                                return (_ref3 = new this.constructor()).with.apply(_ref3, relations).find(this.id);

                            case 4:
                                newThis = _context2.sent;
                                _iteratorNormalCompletion = true;
                                _didIteratorError = false;
                                _iteratorError = undefined;
                                _context2.prev = 8;

                                for (_iterator = relations[Symbol.iterator](); !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                                    r = _step.value;

                                    this[r] = newThis[r];
                                    this.relations[r].loaded = true;
                                }
                                _context2.next = 16;
                                break;

                            case 12:
                                _context2.prev = 12;
                                _context2.t0 = _context2['catch'](8);
                                _didIteratorError = true;
                                _iteratorError = _context2.t0;

                            case 16:
                                _context2.prev = 16;
                                _context2.prev = 17;

                                if (!_iteratorNormalCompletion && _iterator.return) {
                                    _iterator.return();
                                }

                            case 19:
                                _context2.prev = 19;

                                if (!_didIteratorError) {
                                    _context2.next = 22;
                                    break;
                                }

                                throw _iteratorError;

                            case 22:
                                return _context2.finish(19);

                            case 23:
                                return _context2.finish(16);

                            case 24:
                                _context2.next = 49;
                                break;

                            case 26:
                                _context2.prev = 26;
                                _context2.t1 = _context2['catch'](1);

                                console.error('Lazy loading error. Relation \'' + relations + '\' of \'' + this.constructor.name + '\':');
                                console.error(_context2.t1);
                                _iteratorNormalCompletion2 = true;
                                _didIteratorError2 = false;
                                _iteratorError2 = undefined;
                                _context2.prev = 33;
                                for (_iterator2 = relations[Symbol.iterator](); !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                                    _r = _step2.value;

                                    this.relations[_r] = [];
                                }
                                _context2.next = 41;
                                break;

                            case 37:
                                _context2.prev = 37;
                                _context2.t2 = _context2['catch'](33);
                                _didIteratorError2 = true;
                                _iteratorError2 = _context2.t2;

                            case 41:
                                _context2.prev = 41;
                                _context2.prev = 42;

                                if (!_iteratorNormalCompletion2 && _iterator2.return) {
                                    _iterator2.return();
                                }

                            case 44:
                                _context2.prev = 44;

                                if (!_didIteratorError2) {
                                    _context2.next = 47;
                                    break;
                                }

                                throw _iteratorError2;

                            case 47:
                                return _context2.finish(44);

                            case 48:
                                return _context2.finish(41);

                            case 49:
                                return _context2.abrupt('return', this);

                            case 50:
                            case 'end':
                                return _context2.stop();
                        }
                    }
                }, _callee2, this, [[1, 26], [8, 12, 16, 24], [17,, 19, 23], [33, 37, 41, 49], [42,, 44, 48]]);
            }));

            function forceLazyLoad() {
                return _ref2.apply(this, arguments);
            }

            return forceLazyLoad;
        }()

        /**
         * Filter the relations that have already been loaded.
         * @param relations the model's relations
         * @returns {Array} the filtered relations (that haven't been loaded yet)
         */

    }, {
        key: 'filterLoadedRelations',
        value: function filterLoadedRelations() {
            var arr = [];

            for (var _len3 = arguments.length, relations = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
                relations[_key3] = arguments[_key3];
            }

            var _iteratorNormalCompletion3 = true;
            var _didIteratorError3 = false;
            var _iteratorError3 = undefined;

            try {
                for (var _iterator3 = relations[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
                    var r = _step3.value;

                    if (!this.relations[r].loaded) {
                        arr.push(r);
                    }
                }
            } catch (err) {
                _didIteratorError3 = true;
                _iteratorError3 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion3 && _iterator3.return) {
                        _iterator3.return();
                    }
                } finally {
                    if (_didIteratorError3) {
                        throw _iteratorError3;
                    }
                }
            }

            return arr;
        }

        // requests

    }, {
        key: 'request',
        value: function () {
            var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(config) {
                var opt, _response;

                return _regenerator2.default.wrap(function _callee3$(_context3) {
                    while (1) {
                        switch (_context3.prev = _context3.next) {
                            case 0:
                                opt = { method: config.method };


                                if (config.data && !_lodash2.default.isEmpty(config.data)) {
                                    opt.data = config.data;
                                }

                                opt = this.beforeFetch(opt);

                                _context3.prev = 3;
                                _context3.next = 6;
                                return fetch(config.url, opt);

                            case 6:
                                _response = _context3.sent;

                                if (_response.ok) {
                                    _context3.next = 13;
                                    break;
                                }

                                if (!(_response.status >= 500)) {
                                    _context3.next = 10;
                                    break;
                                }

                                throw new _bunchOfExceptions.UnreachableServerException('The server returned HTTP code ' + _response.status + ' (' + _response.statusText + ')');

                            case 10:
                                if (!(_response.status >= 400)) {
                                    _context3.next = 12;
                                    break;
                                }

                                throw new _bunchOfExceptions.InvalidUrlException('The server returned HTTP code ' + _response.status + ' (' + _response.statusText + ')');

                            case 12:
                                throw new _bunchOfExceptions.Exception('Fetch failed with response: ' + _response.statusText);

                            case 13:

                                _response = this.afterFetch(_response);

                                _context3.next = 16;
                                return _response.json();

                            case 16:
                                return _context3.abrupt('return', _context3.sent);

                            case 19:
                                _context3.prev = 19;
                                _context3.t0 = _context3['catch'](3);
                                throw new _bunchOfExceptions.Exception("Fetch failed");

                            case 22:
                            case 'end':
                                return _context3.stop();
                        }
                    }
                }, _callee3, this, [[3, 19]]);
            }));

            function request(_x) {
                return _ref4.apply(this, arguments);
            }

            return request;
        }()
    }, {
        key: 'find',
        value: function () {
            var _ref5 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4(id) {
                var response;
                return _regenerator2.default.wrap(function _callee4$(_context4) {
                    while (1) {
                        switch (_context4.prev = _context4.next) {
                            case 0:
                                this.queryBuilder.awaitType = _QueryBuilder.QUERY_AWAIT_SINGLE;
                                _context4.next = 3;
                                return this.request({
                                    url: '' + this.resourceUrl() + id + this.queryBuilder.getQuery(),
                                    method: _config.REST_CONFIG.http_methods.get
                                });

                            case 3:
                                response = _context4.sent;
                                return _context4.abrupt('return', this.respond(response.data));

                            case 5:
                            case 'end':
                                return _context4.stop();
                        }
                    }
                }, _callee4, this);
            }));

            function find(_x2) {
                return _ref5.apply(this, arguments);
            }

            return find;
        }()
    }, {
        key: 'all',
        value: function () {
            var _ref6 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5() {
                var response;
                return _regenerator2.default.wrap(function _callee5$(_context5) {
                    while (1) {
                        switch (_context5.prev = _context5.next) {
                            case 0:
                                _context5.next = 2;
                                return this.request({
                                    url: '' + this.resourceUrl() + this.queryBuilder.getQuery(),
                                    method: _config.REST_CONFIG.http_methods.get
                                });

                            case 2:
                                response = _context5.sent;
                                return _context5.abrupt('return', this.respond(response.data));

                            case 4:
                            case 'end':
                                return _context5.stop();
                        }
                    }
                }, _callee5, this);
            }));

            function all() {
                return _ref6.apply(this, arguments);
            }

            return all;
        }()
    }, {
        key: 'paginate',
        value: function () {
            var _ref7 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee6() {
                var perPage = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 10;
                var page = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1;
                var response;
                return _regenerator2.default.wrap(function _callee6$(_context6) {
                    while (1) {
                        switch (_context6.prev = _context6.next) {
                            case 0:
                                this.queryBuilder.paginate(perPage, page);

                                _context6.next = 3;
                                return this.request({
                                    url: '' + this.resourceUrl() + this.queryBuilder.getQuery(),
                                    method: _config.REST_CONFIG.http_methods.get
                                });

                            case 3:
                                response = _context6.sent;
                                return _context6.abrupt('return', this.respond(response.data));

                            case 5:
                            case 'end':
                                return _context6.stop();
                        }
                    }
                }, _callee6, this);
            }));

            function paginate() {
                return _ref7.apply(this, arguments);
            }

            return paginate;
        }()
    }, {
        key: 'save',
        value: function () {
            var _ref8 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee7() {
                return _regenerator2.default.wrap(function _callee7$(_context7) {
                    while (1) {
                        switch (_context7.prev = _context7.next) {
                            case 0:
                                this.queryBuilder.awaitType = _QueryBuilder.QUERY_AWAIT_SINGLE;

                                if (!this.hasOwnProperty('id')) {
                                    _context7.next = 3;
                                    break;
                                }

                                return _context7.abrupt('return', this.update());

                            case 3:
                                return _context7.abrupt('return', this.respond(response.data));

                            case 4:
                            case 'end':
                                return _context7.stop();
                        }
                    }
                }, _callee7, this);
            }));

            function save() {
                return _ref8.apply(this, arguments);
            }

            return save;
        }()
    }, {
        key: 'create',
        value: function () {
            var _ref9 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee8() {
                var response;
                return _regenerator2.default.wrap(function _callee8$(_context8) {
                    while (1) {
                        switch (_context8.prev = _context8.next) {
                            case 0:
                                this.queryBuilder.awaitType = _QueryBuilder.QUERY_AWAIT_SINGLE;
                                _context8.next = 3;
                                return this.request({
                                    url: this.resourceUrl(),
                                    method: _config.REST_CONFIG.http_methods.create,
                                    data: this.toJson()
                                });

                            case 3:
                                response = _context8.sent;
                                return _context8.abrupt('return', this.respond(response.data));

                            case 5:
                            case 'end':
                                return _context8.stop();
                        }
                    }
                }, _callee8, this);
            }));

            function create() {
                return _ref9.apply(this, arguments);
            }

            return create;
        }()
    }, {
        key: 'update',
        value: function () {
            var _ref10 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee9() {
                var response;
                return _regenerator2.default.wrap(function _callee9$(_context9) {
                    while (1) {
                        switch (_context9.prev = _context9.next) {
                            case 0:
                                this.queryBuilder.awaitType = _QueryBuilder.QUERY_AWAIT_SINGLE;
                                _context9.next = 3;
                                return this.request({
                                    url: this.links.self,
                                    method: _config.REST_CONFIG.http_methods.update,
                                    data: this.toJson()
                                });

                            case 3:
                                response = _context9.sent;
                                return _context9.abrupt('return', this.respond(response.data));

                            case 5:
                            case 'end':
                                return _context9.stop();
                        }
                    }
                }, _callee9, this);
            }));

            function update() {
                return _ref10.apply(this, arguments);
            }

            return update;
        }()
    }, {
        key: 'delete',
        value: function () {
            var _ref11 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee10() {
                var response;
                return _regenerator2.default.wrap(function _callee10$(_context10) {
                    while (1) {
                        switch (_context10.prev = _context10.next) {
                            case 0:
                                this.queryBuilder.awaitType = _QueryBuilder.QUERY_AWAIT_SINGLE;
                                response = this.request({
                                    url: this.links.self,
                                    method: _config.REST_CONFIG.http_methods.delete
                                });
                                return _context10.abrupt('return', this.respond(response.data));

                            case 3:
                            case 'end':
                                return _context10.stop();
                        }
                    }
                }, _callee10, this);
            }));

            function _delete() {
                return _ref11.apply(this, arguments);
            }

            return _delete;
        }()
    }, {
        key: 'attach',
        value: function () {
            var _ref12 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee11(model) {
                var data = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
                var sync = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
                var queryConfig, response;
                return _regenerator2.default.wrap(function _callee11$(_context11) {
                    while (1) {
                        switch (_context11.prev = _context11.next) {
                            case 0:
                                this.queryBuilder.awaitType = _QueryBuilder.QUERY_AWAIT_SINGLE;
                                queryConfig = {
                                    url: this.getNamespace() + '/' + this.id + '/' + model.getNamespace() + '/' + model.id + (sync ? '?sync=true' : ''),
                                    method: sync ? _config.REST_CONFIG.http_methods.update : _config.REST_CONFIG.http_methods.create
                                };


                                if (data) {
                                    queryConfig.data = data;
                                }

                                _context11.next = 5;
                                return this.request(queryConfig);

                            case 5:
                                response = _context11.sent;
                                return _context11.abrupt('return', this.respond(response.data));

                            case 7:
                            case 'end':
                                return _context11.stop();
                        }
                    }
                }, _callee11, this);
            }));

            function attach(_x5) {
                return _ref12.apply(this, arguments);
            }

            return attach;
        }()
    }, {
        key: 'detach',
        value: function () {
            var _ref13 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee12(model) {
                var response;
                return _regenerator2.default.wrap(function _callee12$(_context12) {
                    while (1) {
                        switch (_context12.prev = _context12.next) {
                            case 0:
                                this.queryBuilder.awaitType = _QueryBuilder.QUERY_AWAIT_SINGLE;
                                _context12.next = 3;
                                return this.request({
                                    url: this.getNamespace() + '/' + this.id + '/' + model.getNamespace() + '/' + model.id,
                                    method: _config.REST_CONFIG.http_methods.delete
                                });

                            case 3:
                                response = _context12.sent;
                                return _context12.abrupt('return', this.respond(response.data));

                            case 5:
                            case 'end':
                                return _context12.stop();
                        }
                    }
                }, _callee12, this);
            }));

            function detach(_x8) {
                return _ref13.apply(this, arguments);
            }

            return detach;
        }()
    }, {
        key: 'sync',
        value: function () {
            var _ref14 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee13(model, data) {
                return _regenerator2.default.wrap(function _callee13$(_context13) {
                    while (1) {
                        switch (_context13.prev = _context13.next) {
                            case 0:
                                this.queryBuilder.awaitType = _QueryBuilder.QUERY_AWAIT_SINGLE;
                                return _context13.abrupt('return', this.attach(model, data, true));

                            case 2:
                            case 'end':
                                return _context13.stop();
                        }
                    }
                }, _callee13, this);
            }));

            function sync(_x9, _x10) {
                return _ref14.apply(this, arguments);
            }

            return sync;
        }()

        // modify query string

    }, {
        key: 'with',
        value: function _with() {
            var _queryBuilder;

            (_queryBuilder = this.queryBuilder).with.apply(_queryBuilder, arguments);

            return this;
        }

        /**
         * Order by a list of fields
         * @param fields the field string, ASC by default. add a '-' ahead to sort in DESC order.
         * EX: model.orderBy('age', '-name') will order by age ASC, and by name DESC
         * @returns {Model} this
         */

    }, {
        key: 'orderBy',
        value: function orderBy() {
            for (var _len4 = arguments.length, fields = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
                fields[_key4] = arguments[_key4];
            }

            var _iteratorNormalCompletion4 = true;
            var _didIteratorError4 = false;
            var _iteratorError4 = undefined;

            try {
                for (var _iterator4 = fields[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
                    var field = _step4.value;

                    var direction = 'ASC';
                    var column = field;
                    if (column[0] === '-') {
                        direction = 'DESC';
                        column = field.substr(1);
                    }

                    this.queryBuilder.orderBy(column, direction);
                }
            } catch (err) {
                _didIteratorError4 = true;
                _iteratorError4 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion4 && _iterator4.return) {
                        _iterator4.return();
                    }
                } finally {
                    if (_didIteratorError4) {
                        throw _iteratorError4;
                    }
                }
            }

            return this;
        }
    }, {
        key: 'limit',
        value: function limit(_limit, offset) {
            this.queryBuilder.limit(_limit, offset);

            return this;
        }
    }, {
        key: 'orderByDesc',
        value: function orderByDesc(column) {
            return this.orderBy('-' + column);
        }
    }, {
        key: 'where',
        value: function where(key, operator) {
            var value = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

            this.queryBuilder.where(key, operator, value);

            return this;
        }
    }, {
        key: 'from',
        value: function from(date) {
            this.queryBuilder(date);

            return this;
        }
    }, {
        key: 'to',
        value: function to(date) {
            this.queryBuilder.to(date);

            return this;
        }
    }, {
        key: 'select',
        value: function select() {
            var _queryBuilder2;

            (_queryBuilder2 = this.queryBuilder).select.apply(_queryBuilder2, arguments);

            return this;
        }
    }, {
        key: 'distinct',
        value: function distinct() {
            var bool = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;

            this.queryBuilder.distinct(bool);

            return this;
        }

        // build model

        /**
         * Make changes to the received json before the parsing
         * @param json the json received
         * @returns the computed json
         */

    }, {
        key: 'computeJsonBeforeParsing',
        value: function computeJsonBeforeParsing(json) {
            return json;
        }
    }, {
        key: 'respond',
        value: function respond(json) {
            json = this.computeJsonBeforeParsing(json);
            return _ResponseHandler2.default.ofJson(this, json);
        }

        // lib

    }, {
        key: 'resourceUrl',
        value: function resourceUrl() {
            return this.getBaseUrl() + '/' + this.getNamespace() + '/';
        }
    }, {
        key: 'isCollection',
        value: function isCollection(data) {
            return _lodash2.default.isArray(data.data);
        }
    }, {
        key: 'selfValidate',
        value: function selfValidate() {
            var name = this.getNamespace();

            if (name === null || !_lodash2.default.isString(name) || name.length === 0) {
                throw new Error('LRA Query Builder: Resource name not defined in ' + this.constructor.name + ' model. Implement resourceName method in the ' + this.constructor.name + ' model to resolve this error.');
            }
        }
    }]);
    return Model;
}();

exports.default = Model;